#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Session 26/6/2017 - machine learning
# 
# Découverte des trois problèmes de machine learning exposé dans l'article [Machine Learning - session 6](http://www.xavierdupre.fr/app/actuariat_python/helpsphinx/blog/2017/2017-06-25_plan_session6.html).

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Problème 1 : comparaison random forest, linéaire
# 
# C'est un problème de régression. On cherche à comparer une random forest avec un modèle linéaire.
# 
# * Comparaison des tests de coefficients pour un modèle linéaire
#   [OLS](http://www.statsmodels.org/dev/generated/statsmodels.regression.linear_model.OLS.html>)
#   et des [features importance](http://scikit-learn.org/stable/auto_examples/ensemble/plot_forest_importances.html)
# * Résultat au niveau d'une observation [treeinterpreter](https://github.com/andosa/treeinterpreter>)
# * Données : [Housing](https://archive.ics.uci.edu/ml/datasets/housing>), [Forest Fire](https://archive.ics.uci.edu/ml/datasets/Forest+Fires)

# ### Données

# In[2]:


import pandas
df = pandas.read_csv("data/housing.data", delim_whitespace=True, header=None)
df.head()


# In[3]:


cols = "CRIM ZN INDUS CHAS NOX RM AGE DIS RAD TAX PTRATIO B LSTAT MEDV".split()
df.columns = cols
df.head()


# In[4]:


X = df.drop("MEDV", axis=1)
y = df["MEDV"]
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33)


# ### Random Forest

# In[5]:


from sklearn.ensemble import RandomForestRegressor
clr = RandomForestRegressor()
clr.fit(X, y)


# In[6]:


importances = clr.feature_importances_
importances


# On s'inspire de l'exemple [Feature importances with forests of trees](http://scikit-learn.org/stable/auto_examples/ensemble/plot_forest_importances.html).

# In[7]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[8]:


import matplotlib.pyplot as plt
import numpy as np
plt.figure(figsize=(12,4))
indices = np.argsort(importances)[::-1]
std = np.std([tree.feature_importances_ for tree in clr.estimators_],
             axis=0)
plt.title("Feature importances")
plt.bar(range(X.shape[1]), importances[indices],
        color="r", yerr=std[indices], align="center")
xlabels = list(df.columns[:-1])
xlabels = [xlabels[i] for i in indices]
plt.xticks(range(X.shape[1]), xlabels)
plt.xlim([-1, X.shape[1]])
plt.show()


# In[9]:


from sklearn.metrics import r2_score
r2_score(y_train, clr.predict(X_train))


# In[10]:


r2_score(y_test, clr.predict(X_test))


# ### Modèle linéaire

# In[11]:


import statsmodels.api as sm


# In[12]:


model = sm.OLS(y_train, X_train)


# In[13]:


results = model.fit()


# In[14]:


results.params


# In[15]:


results.summary()


# In[16]:


model = sm.OLS(y,X.drop("LSTAT", axis=1))
results = model.fit()
results.summary()


# ### TPOT
# 
# [TPOT](https://github.com/rhiever/tpot) est un module d'apprentissage automatique.

# In[17]:


try:
    from tpot import TPOTRegressor
except ImportError:
    # for sklearn 0.22
    import sklearn.preprocessing    
    from sklearn.impute import SimpleImputer
    sklearn.preprocessing.Imputer = SimpleImputer
    from tpot import TPOTRegressor

tpot = TPOTRegressor(generations=2, population_size=50, verbosity=2)
tpot.fit(X_train, y_train)
print(tpot.score(X_test, y_test))
tpot.export('tpot_boston_pipeline.py')


# Le module optimise les hyperparamètres, parfois un peu trop à en juger la mauvaise performance obtenue sur la base de test.

# In[18]:


r2_score(y_train, tpot.predict(X_train))


# In[19]:


r2_score(y_test, tpot.predict(X_test))


# ### Feature importance pour une observations
# 
# On reprend la première random forest et on utilise le module [treeinterpreter](https://github.com/andosa/treeinterpreter).

# In[20]:


clr = RandomForestRegressor()
clr.fit(X, y)


# In[21]:


from treeinterpreter import treeinterpreter as ti
prediction, bias, contributions = ti.predict(clr, X_test)


# In[22]:


for i in range(min(2, X_train.shape[0])):
    print("Instance", i)
    print("Bias (trainset mean)", bias[i])
    print("Feature contributions:")
    for c, feature in sorted(zip(contributions[i], df.columns), 
                             key=lambda x: -abs(x[0])):
        print(feature, round(c, 2))
    print( "-"*20)


# ## Problème 2 : série temporelle
# 
# On prend une série sur [Google Trends](https://trends.google.fr/trends/explore?q=tennis%20live), dans notre cas, c'est la requête *tennis live*. On compare une approche linéaire et une approche non linéaire.

# ### Approche linéaire

# In[23]:


import pandas
df = pandas.read_csv("data/multiTimeline.csv", skiprows=1)


# In[24]:


df.columns= ["Semaine", "compte"]


# In[25]:


df["SemaineDt"] = pandas.to_datetime(df.Semaine)


# In[26]:


df=df.set_index("SemaineDt")


# In[27]:


df["compte"] = df["compte"].astype(float)


# In[28]:


df.head()


# In[29]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[30]:


df.plot()


# In[31]:


from statsmodels.tsa.arima_model import ARIMA
arma_mod = ARIMA(df["compte"].values, order=(6 ,1, 1))
res = arma_mod.fit()
res.params


# In[32]:


res.summary()


# ### Méthode non linéaire
# 
# On construire la matrice des séries décalées. Cette méthode permet de sortir du cadre linéaire et d'ajouter d'autres variables.

# In[33]:


from statsmodels.tsa.tsatools import lagmat
lag = 8
X = lagmat(df["compte"], lag)
lagged = df.copy()
for c in range(1,lag+1):
    lagged["lag%d" % c] = X[:, c-1]
pandas.concat([lagged.head(), lagged.tail()])


# In[34]:


xc = ["lag%d" % i for i in range(1,lag+1)]
split = 0.66
isplit = int(len(lagged) * split)
xt = lagged[10:][xc]
yt = lagged[10:]["compte"]
X_train, y_train, X_test, y_test = xt[:isplit], yt[:isplit], xt[isplit:], yt[isplit:]


# In[35]:


from sklearn.ensemble import RandomForestRegressor
clr = RandomForestRegressor()
clr.fit(X_train, y_train)


# In[36]:


from sklearn.metrics import r2_score
r2 = r2_score(y_test.values, clr.predict(X_test))
r2


# In[37]:


plt.scatter(y_test.values, clr.predict(X_test));


# ## Texte
# 
# On cherche à comparer une [LDA](http://scikit-learn.org/stable/modules/generated/sklearn.decomposition.LatentDirichletAllocation.html) avec [word2vec](https://radimrehurek.com/gensim/models/word2vec.html) et [kmeans](http://scikit-learn.org/stable/modules/generated/sklearn.cluster.KMeans.html) et les données qui sont sur [ensae_teaching_cs/src/ensae_teaching_cs/data/data_web/](https://github.com/sdpython/ensae_teaching_cs/tree/master/src/ensae_teaching_cs/data/data_web).

# In[38]:


from ensae_teaching_cs.data import twitter_zip
df = twitter_zip(as_df=True)
df.head(n=2).T


# ### Des mots aux coordonnées - tf-idf

# In[39]:


keep = df.text.dropna().index


# In[40]:


dfnonan = df.iloc[keep, :]
dfnonan.shape


# In[41]:


from sklearn.feature_extraction.text import TfidfVectorizer
tfidf_vectorizer = TfidfVectorizer(max_df=0.95, min_df=2, max_features=1000)
tfidf = tfidf_vectorizer.fit_transform(dfnonan["text"])


# In[42]:


tfidf[:2, :]


# In[43]:


tfidf[:2, :].todense()


# ### LDA

# In[44]:


from sklearn.decomposition import LatentDirichletAllocation
lda = LatentDirichletAllocation(n_components=10, max_iter=5,
                                learning_method='online',
                                learning_offset=50.,
                                random_state=0)


# In[45]:


lda.fit(tfidf)


# In[46]:


tf_feature_names = tfidf_vectorizer.get_feature_names()
tf_feature_names[100:103]


# In[47]:


lda.components_.shape


# On obtient dix vecteurs qui représentent les dix vecteurs associés aux dix clusters. Chaque dimension relié au fait que le mot appartient ou non au cluster.

# In[48]:


def print_top_words(model, feature_names, n_top_words):
    for topic_idx, topic in enumerate(model.components_):
        print("Topic #%d:" % topic_idx)
        print(" ".join([feature_names[i]
                        for i in topic.argsort()[:-n_top_words - 1:-1]]))
    print()


# In[49]:


print_top_words(lda, tf_feature_names, 10)


# ### Clustering

# In[50]:


from sklearn.cluster import KMeans
km = KMeans(n_clusters=10)
km.fit(tfidf)


# In[51]:


km.cluster_centers_.shape


# In[52]:


def print_top_words(model, feature_names, n_top_words):
    for topic_idx, topic in enumerate(model.cluster_centers_):
        print("Topic #%d:" % topic_idx)
        print(" ".join([feature_names[i]
                        for i in topic.argsort()[:-n_top_words - 1:-1]]))
    print()


# In[53]:


print_top_words(km, tf_feature_names, 10)


# Ah les stop words....

# In[54]: